#!/bin/sh
#
# © Copyright 2015 Hewlett Packard Enterprise Development LP..
#
# See "man chkconfig" for information on next two lines (Red Hat only)
# chkconfig: 2345 91 2
# description: HPE Advanced Server Recovery Daemon
#
#
# Following lines are in conformance with LSB 1.2 spec
### BEGIN INIT INFO
# Provides:            hp-asrd
# Required-Start:      hp-health
# Required-Stop:
# Default-Start:       2 3 4 5
# Default-Stop:        0 1 6
# Description:         HPE Advanced Server Recovery Daemon
### END INIT INFO

##
## By default, this script will use the ASR settings from the system ROM.
## You can configure these values using either the hpasmcli command, or by using hpsmh or by
## entering the ROM-Based Setup Utility (RBSU) during boot.
##


# WATCHDOG_PING_FREQUENCY
#  The number of seconds between attempts to reset the watchdog
WATCHDOG_PING_RATE=1

#### YOU SHOULD NOT NEED TO MODIFY ANYTHING BELOW THIS LINE ####

PATH="/sbin:/usr/sbin:/bin:/usr/bin"

if [ -f /lib/lsb/init-functions ]; then
    . /lib/lsb/init-functions || exit 1
else
    # Implement our own versions of the LSB-defined functions.
    # This is not an LSB-compliant implementation, it just provides 
    # the pieces we use.
    if [ -f /etc/rc.d/init.d/functions ]; then
	. /etc/rc.d/init.d/functions
    fi
    pidofproc() {
	pidof "$1"
    }
    echo "hpasrd 1"
    start_daemon() {
	local daemon="$1"
	shift
	local args="$*"
	if [ -n "$(pidofproc $daemon)" ]; then
	    return 0
	fi
	$daemon $args
	if [ -n "$(pidofproc $daemon)" ]; then
	    return 0
	else
	    return 1
	fi
    }
    killproc() {
	local daemon="$1"
	local signal=""
	local pid="$(pidofproc $daemon)"
	if [ "$#" -eq 2 ]; then
	    signal="-s $(echo $2 | sed -r 's/-(SIG)?(.*)/\2/')"
	fi
	if [ -n "$pid" ]; then
	    kill $signal "$pid"
	fi
	return 0
    }
    _log_msg() {
	printf "$*\n"
    }
    log_success_msg() {
	if [ -f /etc/rc.d/init.d/functions ]; then
	    echo_success
	fi
	_log_msg "$*"
    }
    log_failure_msg() {
	if [ -f /etc/rc.d/init.d/functions ]; then
	    echo_failure
	fi
	_log_msg "$*"
    }
fi

HPHEALTH_BIN="/opt/hp/hp-health/bin"
DESCRIPTION="HP Advanced Server Recovery Daemon"
DAEMON="$HPHEALTH_BIN/hp-asrd"
HPASMCLI="/sbin/hpasmcli"
PIDFILE="/var/run/hp-asrd.pid"
RETVAL=0

do_udev_settle() {
    if [ -x /sbin/udevadm ]; then
	/sbin/udevadm settle
    elif [ -x /sbin/udevsettle ]; then
	/sbin/udevsettle
    fi
}

have_watchdog() {
    do_udev_settle
    if [ -d /sys/class/misc/watchdog ] && [ -e /dev/watchdog ]; then
	return 0
    fi
    return 1
}

load_modules() {
    if have_watchdog; then
	# There is already a watchdog loaded, hopefully it is
	# one that we support
	return 0
    fi
    modprobe hpwdt > /dev/null 2>&1
    if have_watchdog; then
	return 0
    fi
    # No iLO2/iLO3 (or no hpwdt available)
    modprobe -r hpwdt > /dev/null 2>&1
    modprobe ipmi_watchdog > /dev/null 2>&1
    if have_watchdog; then
	return 0
    fi
    # Unsupported platform?
    modprobe -r ipmi_watchdog > /dev/null 2>&1
    return 1
}

PCI_VENDOR_ID_COMPAQ="0e11"
PCI_VENDOR_ID_HP="103c"
PCI_DEVICE_ID_ILO="b203" # iLO and iLO2
PCI_DEVICE_ID_ILO3="3307"

PCIID_ILO="$PCI_VENDOR_ID_COMPAQ:$PCI_DEVICE_ID_ILO"
PCIID_ILO3="$PCI_VENDOR_ID_HP:$PCI_DEVICE_ID_ILO3"

# return codes for probe_ilo()
ILO_NO_ILO=0
ILO_NO_EMBEDDED_HEALTH=1
ILO_WITH_EMBEDDED_HEALTH=2
ILO_WITH_EMBEDDED_HEALTH_G3=3

## Scan the system for an iLO device. Return one of the following:
## $ILO_NO_ILO - no iLO device found
## $ILO_NO_EMBEDDED_HEALTH - iLO found, does not support embedded health
## $ILO_WITH_EMBEDDED_HEALTH - iLO supporting embedded health found
## $ILO_WITH_EMBEDDED_HEALTH_G3 - iLO3 supporting embedded health found
probe_ilo() {
    local dev=""

    for id in $PCIID_ILO $PCIID_ILO3; do
    dev="$(lspci -d $id | cut -d' ' -f1)"
    if [ -n "$dev" ]; then
        if [ "$id" = "$PCIID_ILO" ]; then
            if ! lspci -xv -s "$dev" | grep -q "3c 10 05 33"; then
		return "$ILO_NO_EMBEDDED_HEALTH"
            fi
            return "$ILO_WITH_EMBEDDED_HEALTH"
        fi
        return "$ILO_WITH_EMBEDDED_HEALTH_G3"
    fi
    done

    return "$ILO_NO_ILO"
}

have_ilo() {
    probe_ilo
    if [ $? = "$ILO_NO_ILO" ]; then
	return 1
    fi
    return 0
}

start() {
	if ! have_ilo; then
	    return 0
	fi

	load_modules # Ignore return code; iloraw mode may still work

	start_daemon $DAEMON -p "$WATCHDOG_PING_RATE"

	pid=$(pidof $DAEMON)
	if [ -n "$pid" ]; then
	    echo "$pid" > "$PIDFILE"
	    log_success_msg "Starting $DESCRIPTION"
	else
	    log_failure_msg "Starting $DESCRIPTION"
	fi
}

stop() {
	if ! have_ilo; then
	    return 0
	fi
	pid="$(pidof $DAEMON)"
	if [ -z "$pid" ]; then
	    log_success_msg "hp-asrd not running"
	    return 0
	fi

	kill -TERM $pid
	printf "Waiting for hp-asrd[$pid] to terminate"
	while [ -d "/proc/$pid/" ]; do
	    printf "."
	    sleep 1
	done
	printf "\n"
	rm -f "$PIDFILE"
	log_success_msg "$DESCRIPTION Terminated"
}

case "$1" in
    start)
	start
	exit $?
	;;
    stop)
	stop
	exit $?
	;;
    restart|force-restart)
	stop
	start
	exit $?
	;;
    status)
	
	pid=$(pidof $DAEMON)
	RETVAL=$?
	if [ -n "$pid" ]; then
	    log_success_msg "Running"
	else
	    log_success_msg "Not Running"
	    RETVAL=3
	fi
	exit $RETVAL 
	;;
    *)
	cmaecho "Usage: $0 {start|stop|restart|status}"
	exit 1
        ;;
esac
